/***********************************************************************************
 * LabTestSim - Laboratory Test Simulator using Elastoplastic models               *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>     *
 *                                                                                 *
 * This file is part of LabTestSim.                                                *
 *                                                                                 *
 * LabTestSim is free software; you can redistribute it and/or modify it under the *
 * terms of the GNU General Public License as published by the Free Software       *
 * Foundation; either version 2 of the License, or (at your option) any later      *
 * version.                                                                        *
 *                                                                                 *
 * LabTestSim is distributed in the hope that it will be useful, but WITHOUT ANY   *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.        *
 *                                                                                 *
 * You should have received a copy of the GNU General Public License along with    *
 * LabTestSim; if not, write to the Free Software Foundation, Inc., 51 Franklin    *
 * Street, Fifth Floor, Boston, MA 02110-1301, USA                                 *
 ***********************************************************************************/

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <vector>
#include <algorithm>
#include "readmat.h"

// external
#include "mechsys/parameters/parameters.h"

ReadMat::ReadMat(std::string XMATERIALS_FileName) : _is_mat_file_ok(false)
{
	std::ifstream file(XMATERIALS_FileName.c_str(),std::ios::in);
	
	if (file)
	{
		// Heading
		std::string head;
		std::getline(file,head);
    
		// Data
		std::string key;
		std::string desc;
		double      lam;
		double      kap;
		double      nuu;
		double      rcs;
		double      enc;
		double      pnc;
		double      bet;
		double      cc1;
		double      cc2;
		double      dd1;
		double      dd2;
		_num_mats = 0;
		while (file>>key>>desc>>lam>>kap>>nuu>>rcs>>enc>>pnc>>bet>>cc1>>cc2>>dd1>>dd2)
		{
			_key.push_back(key);
			_desc.push_back(desc);
			_lam.push_back(lam); // 1
			_kap.push_back(kap); // 2
			_nuu.push_back(nuu); // 3
			_rcs.push_back(rcs); // 4
			_enc.push_back(enc); // 5
			_pnc.push_back(pnc); // 6
			_bet.push_back(bet); // 7
			_cc1.push_back(cc1); // 8
			_cc2.push_back(cc2); // 9
			_dd1.push_back(dd1); // 10
			_dd2.push_back(dd2); // 11
			_num_mats++;
		}
		_is_mat_file_ok = true;
		file.close();
	}
	else
	{
		_errors.append("Could not open xmaterials.dat <");
		_errors.append(XMATERIALS_FileName);
		_errors.append("> \n");
		_is_mat_file_ok = false;
	}
}

bool ReadMat::SetPrms(MechSys::Parameters * prms, std::string MatKey)
{
	if (!_is_mat_file_ok) return false;

	std::vector<std::string>::iterator loc;
	loc = std::find(_key.begin(),_key.end(),MatKey);
	if (loc==_key.end()) return false;

	int locint = (loc - _key.begin());

	prms->Setlam (_lam[locint]); // 1
	prms->Setkap (_kap[locint]); // 2
	prms->Setnu  (_nuu[locint]); // 3
	prms->Setrcs (_rcs[locint]); // 4
	prms->Setenc (_enc[locint]); // 5
	prms->Setpnc (_pnc[locint]); // 6
	prms->Setbet (_bet[locint]); // 7
	prms->Setc1  (_cc1[locint]); // 8
	prms->Setc2  (_cc2[locint]); // 9
	prms->Setd1  (_dd1[locint]); // 10
	prms->Setd2  (_dd2[locint]); // 11
	
	prms->CalculateM();
	prms->CalculateMus();

	return true;
}

std::ostream & operator<< (std::ostream & os, ReadMat & rm)
{
	//          1   2   3   4   5   6   7   8   9  10  11
	os << " key    lam     kap      nuu     rcs     enc     pnc     bet     cc1     cc2     dd1     dd2\n";
	for (int i=0; i<static_cast<int>(rm._key.size()); ++i)
	{
		os << std::setw(4) << rm._key[i]
		   << std::setw(8) << rm._lam[i]
		   << std::setw(8) << rm._kap[i]
		   << std::setw(8) << rm._nuu[i]
		   << std::setw(8) << rm._rcs[i]
		   << std::setw(8) << rm._enc[i]
		   << std::setw(8) << rm._pnc[i]
		   << std::setw(8) << rm._bet[i]
		   << std::setw(8) << rm._cc1[i]
		   << std::setw(8) << rm._cc2[i]
		   << std::setw(8) << rm._dd1[i]
		   << std::setw(8) << rm._dd2[i]
		   << std::endl;
	}
	return os;
}
