/***********************************************************************************
 * LabTestSim - Laboratory Test Simulator using Elastoplastic models               *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>     *
 *                                                                                 *
 * This file is part of LabTestSim.                                                *
 *                                                                                 *
 * LabTestSim is free software; you can redistribute it and/or modify it under the *
 * terms of the GNU General Public License as published by the Free Software       *
 * Foundation; either version 2 of the License, or (at your option) any later      *
 * version.                                                                        *
 *                                                                                 *
 * LabTestSim is distributed in the hope that it will be useful, but WITHOUT ANY   *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.        *
 *                                                                                 *
 * You should have received a copy of the GNU General Public License along with    *
 * LabTestSim; if not, write to the Free Software Foundation, Inc., 51 Franklin    *
 * Street, Fifth Floor, Boston, MA 02110-1301, USA                                 *
 ***********************************************************************************/

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <iostream>
#include <stdexcept>
#include <gtkmm/main.h>
#include "../inputdata.h"
#include "gui.h"
#include "mywindow.h"

#ifdef GLADEFILEDIR
# define GLADEFILE GLADEFILEDIR"/gui.glade"
#else
# define GLADEFILE "gui.glade"
#endif

Gui::Gui(int argc, char **argv,
		 bool (*pRunSim)(InputData & ID), 
		 bool (*pGenPltFiles)(InputData & ID))
	: _pRunSim(pRunSim),
	  _pGenPltFiles(pGenPltFiles),
	  _pWin(0)
{
	Gtk::Main Kit(argc, argv);
	try
	{
		// Load the Glade file and instantiate its widgets
		Glib::RefPtr<Gnome::Glade::Xml> refXml;

		std::cout << "GLADEFILE = ";
		std::cout << GLADEFILE << std::endl;
		
		refXml = Gnome::Glade::Xml::create(GLADEFILE);
		// Get the Glade-instantiated dialog
		try
		{
			refXml->get_widget_derived("main_window", _pWin);
			// Show window
			if(_pWin)
			{
				_pWin->SetFunPtrs(_pRunSim,_pGenPltFiles);
				Kit.run(*_pWin);
			}
		}
		catch(char * res)
		{
			std::cout << res << std::endl;
		}
		catch(std::exception & e)
		{
			std::cout << e.what() << std::endl;
		}
	}
	catch(const Gnome::Glade::XmlError & ex)
	{
		std::cerr << ex.what() << std::endl;
	}
}
