/*************************************************************************************
 * MechSys - A C++ library to simulate (Continuum) Mechanical Systems                *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of MechSys.                                                     *
 *                                                                                   *
 * MechSys is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

#include <iostream>
#include <cmath>
#include "stressinvs.h"
#include "charinvs.h"

namespace MechSys
{

bool StressInvs::Get_p_q(double & p, double & q)
{
	double sq2 = sqrt(2.0);
	double   s = pow(_sig[0]-_sig[1],2.0)
		       + pow(_sig[1]-_sig[2],2.0)
			   + pow(_sig[2]-_sig[0],2.0)
		       + 6.0*pow(_sig[3]/sq2,2.0)
			   + 6.0*pow(_sig[4]/sq2,2.0)
			   + 6.0*pow(_sig[5]/sq2,2.0);
	p = (_sig[0]+_sig[1]+_sig[2])/3.0;
	q = sqrt(s/2.0);
	return true;
}

bool StressInvs::Get_tn_ts_rz(double & tn, double & ts, double & rz)
{
	double ZERO=1.0e-14;

	CharInvs ci(_sig);
	if (fabs(ci.I(1))<=ZERO)
	{
		_errors.append("StressInvs::Get_tn_ts_rz:: fabs(ci.I(1)) failed !\n");
		return false;
	}
	double s = ci.I(0)*ci.I(1)*ci.I(2)-9.0*pow(ci.I(2),2.0);
	if (s<ZERO)
	{
		if (s<0.0) s = 0.0;
		else
		{
			_errors.append("StressInvs::Get_tn_ts_rz:: fabs(s)<0.0) failed !\n");
			return false;
		}
	}
	rz = ci.I(2)/ci.I(1);
	if (rz<0.0)
	{
		_errors.append("StressInvs::Get_tn_ts_rz:: rz<0.0 failed !\n");
		return false;
	}
	tn = 3.0*rz;
	ts = sqrt(s)/ci.I(1);
	if (ts!=ts)
	{
		_errors.append("StressInvs::Get_tn_ts_rz:: ts!=ts failed !\n");
		return false;  // avoid NaN values
	}
	rz = sqrt(rz);
	return true;
}

}; // namespace MechSys
