/*************************************************************************************
 * MechSys - A C++ library to simulate (Continuum) Mechanical Systems                *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of MechSys.                                                     *
 *                                                                                   *
 * MechSys is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

#ifndef STRESSINVS_H
#define STRESSINVS_H

#include <string>
#include "../base/mechsys.h"

namespace MechSys
{

class StressInvs
{
public:
	StressInvs(const Tensr2 & Sig) : _sig(Sig) {}
	void Reset(const Tensr2 & Sig) { _sig = Sig; }

	bool Get_p_q      (double & p , double & q);
	bool Get_tn_ts_rz (double & tn, double & ts, double & rz);

	std::string GetErrors() const { return _errors; }
	
private:
	Tensr2 _sig;
	// p  Cambridge Mean stress
	// q  Cambridge Deviatoric stress
	// tn Nakai's Mean stress
	// ts Nakai's Deviatoric stress
	// rz = sqrt(I3/I2)
	std::string _errors;

}; // StressInvs

}; // namespace MechSys

#endif
