/*************************************************************************************
 * MechSys - A C++ library to simulate (Continuum) Mechanical Systems                *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of MechSys.                                                     *
 *                                                                                   *
 * MechSys is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

#ifndef LAWINTEG_H
#define LAWINTEG_H

#include "../base/mechsys.h"
#include "../models/intvars.h"

namespace MechSys
{

template<typename ModelType>
class LawIntegScheme
{
public:
	LawIntegScheme() : _num_calls(0) {}
	virtual bool     StressUpdate(ModelType * model, const Tensr2 & Deps, Tensr2 & Dsig)=0;
	virtual SchemeID GetID() const =0;
	int              GetNumCalls() { return _num_calls; }
	double           CalcNormS(const Tensr2 & sig, const IntVars & z) const;
	void             ResetStat() { _num_calls = 0; }
protected:
	int _num_calls; // number of calls to a scheme
}; // LawIntegScheme

}; // namespace MechSys

#endif
