/*************************************************************************************
 * MechSys - A C++ library to simulate (Continuum) Mechanical Systems                *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of MechSys.                                                     *
 *                                                                                   *
 * MechSys is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

#ifndef MECHSYS_MODEL_H
#define MECHSYS_MODEL_H

#include <iostream>
#include <sstream>
#include "../base/mechsys.h"
#include "../parameters/parameters.h"

namespace MechSys
{

class Model
{

public:
	Model();
	
	virtual bool      Set          (Parameters const * Prms, const Tensr2 & Sig) =0;
	virtual bool      Reset        (const Tensr2 & Sig)                          =0;
	virtual bool      Actualize    (const Tensr2 & Dsig, Tensr2 & Deps)          =0; // Return Deps
	virtual bool      StressUpdate (const Tensr2 & Deps, Tensr2 & Dsig)          =0; // Return Dsig
	virtual Tensr4  * TgCompliance ()                                            =0; // C
	virtual Tensr4  * TgStiffness  ()                                            =0; // D
	virtual void      Out          (std::ostringstream & oss) const              =0;
	virtual ModelID   GetID        ()                         const              =0;

protected:
	Parameters const * _prms;  // Material parameters
	Tensr2             _I;     // Second order identity (Kronecker)
	Tensr4             _IdyI;  // I dyadic I
	Tensr4             _IIsym; // Fourth order symmetric identity

}; // class Model

}; // namespace MechSys

std::ostream & operator<< (std::ostream & os, const MechSys::Model & mo);

#endif
