/*************************************************************************************
 * MechSys - A C++ library to simulate (Continuum) Mechanical Systems                *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of MechSys.                                                     *
 *                                                                                   *
 * MechSys is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

#ifndef MECHSYS_NLELASTIC_H
#define MECHSYS_NLELASTIC_H

#include <sstream>
#include "../base/mechsys.h"
#include "../parameters/parameters.h"
#include "../lawinteg/lawinteg.h"
#include "model.h"

namespace MechSys
{

template<typename ModelType> class SchForwardEuler;
template<typename ModelType> class SchModifiedEuler;
template<typename ModelType> class SchBackwardEuler;

class NlElastic : public Model // Non-Linear Elastic
{

	friend class SchForwardEuler <NlElastic>;
	friend class SchModifiedEuler<NlElastic>;
	friend class SchBackwardEuler<NlElastic>;

public:

	bool     Set          (Parameters const * Prms, const Tensr2 & Sig);
	bool     Reset        (const Tensr2 & Sig);
	bool     Actualize    (const Tensr2 & Dsig, Tensr2 & Deps); // return Deps
	bool     StressUpdate (const Tensr2 & Deps, Tensr2 & Dsig); // return Dsig
	Tensr4 * TgCompliance (); // C
	Tensr4 * TgStiffness  (); // D
	void     Out          (std::ostringstream & oss) const;
	ModelID  GetID        ()                         const { return nlelastic; }

	// Set
	void SetScheme(LawIntegScheme<NlElastic> * Scheme)
	{
		       _scheme = Scheme;
		_is_scheme_set = true;
	}

private:

	// Constants
	Tensr4 _D;
	Tensr4 _C;
	double _ce;
	// Always up-to-date
	Tensr2 _sig;
	double _p;
	double _E;
	// To be calculated
	Tensr4 _De;
	Tensr4 _Ce;
	// Scheme
	LawIntegScheme<NlElastic> * _scheme;
	bool                        _is_scheme_set;

	bool _update_state();
	
}; // class nlElastic
	
}; // namespace MechSys

#endif
