/*************************************************************************************
 * MechSys - A C++ library to simulate (Continuum) Mechanical Systems                *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of MechSys.                                                     *
 *                                                                                   *
 * MechSys is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

#ifndef MECHSYS_SUBCAM_H
#define MECHSYS_SUBCAM_H

#include "../base/mechsys.h"
#include "genericep.h"

namespace MechSys
{

class SubCam : public GenericEP
{
public:

	SubCam(SubCamSurfType SurfType);
	ModelID GetID() const { return subcam; };
	
private:

	// Constants
	SubCamSurfType _surf_type;
	double         _chi;
	// Always up-to-date
	double         _p;
	double         _q;
	double         _G;
	double         _tr_r;
	
	// Methods inherited from GenericEP
	bool _model_specific_set();
	bool _force_zalpha(Tensr2 & Sig);
	bool _start_intvars(Tensr2 & Sig);
	bool _model_specific_update();
	bool _calculate_gradients();
	bool _calculate_hardening();
	void _model_specific_out(std::ostringstream & oss) const;

}; // class SubCam

}; // namespace MechSys

#endif
