#####################################################################################
# MechSys - A C++ library to simulate (Continuum) Mechanical Systems                #
# Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       #
#                                                                                   #
# This file is part of MechSys.                                                     #
#                                                                                   #
# MechSys is free software; you can redistribute it and/or modify it under the      #
# terms of the GNU General Public License as published by the Free Software         #
# Foundation; either version 2 of the License, or (at your option) any later        #
# version.                                                                          #
#                                                                                   #
# MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   #
# PARTICULAR PURPOSE. See the GNU General Public License for more details.          #
#                                                                                   #
# You should have received a copy of the GNU General Public License along with      #
# MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, #
# Fifth Floor, Boston, MA 02110-1301, USA                                           #
#####################################################################################

CC      = g++
CFLAGS  = -g -Wall
#CFLAGS  = -O2 -DNDEBUG
PAT     = ../src

BASE    = $(PAT)/base
INVS    = $(PAT)/invariants
LAWS    = $(PAT)/lawinteg
LINA    = $(PAT)/linalg
MODS    = $(PAT)/models
PRMS    = $(PAT)/parameters

INCS    = -I$(HOME)/opt/include -I$(PAT)
LIBS    = -L$(HOME)/opt/lib

B_DEPS_ = mechsys.h
B_DEPS  = $(patsubst %,$(BASE)/%,$(B_DEPS_))
B_XOBJS = $(patsubst %,$(BASE)/%,$(B_DEPS_:.h=.o))

I_DEPS_ = charinvs.h straininvs.h stressinvs.h
I_DEPS  = $(patsubst %,$(INVS)/%,$(I_DEPS_))
I_XOBJS = $(patsubst %,$(INVS)/%,$(I_DEPS_:.h=.o))

L_DEPS_ = lawinteg.h feuler.h meuler.h
L_DEPS  = $(patsubst %,$(LAWS)/%,$(L_DEPS_))
L_XOBJS = $(patsubst %,$(LAWS)/%,$(L_DEPS_:.h=.o))

N_DEPS_ = jacobirot.h matrix.h
N_DEPS  = $(patsubst %,$(LINA)/%,$(N_DEPS_))
N_XOBJS = $(patsubst %,$(LINA)/%,$(N_DEPS_:.h=.o))

M_DEPS_ = genericep.h intvars.h model.h nlelastic.h subcam.h subtij.h tijtensor.h
M_DEPS  = $(patsubst %,$(MODS)/%,$(M_DEPS_))
M_XOBJS = $(patsubst %,$(MODS)/%,$(M_DEPS_:.h=.o))

P_DEPS_ = parameters.h
P_DEPS  = $(patsubst %,$(PRMS)/%,$(P_DEPS_))
P_XOBJS = $(patsubst %,$(PRMS)/%,$(P_DEPS_:.h=.o))

PROG  = test
SRCS  = test.cpp
OBJS  = $(SRCS:.cpp=.o)

$(PROG): $(OBJS) B_subsys I_subsys L_subsys N_subsys M_subsys P_subsys
	$(CC) $(OBJS) $(B_XOBJS) $(I_XOBJS) $(L_XOBJS) $(N_XOBJS) $(M_XOBJS) $(P_XOBJS) $(LIBS) $(LDFLAGS) -o $@

%.o: %.cpp $(B_DEPS) $(I_DEPS) $(L_DEPS) $(N_DEPS) $(M_DEPS) $(P_DEPS)
	$(CC) $(CFLAGS) $(INCS) -c $<

B_subsys:
	cd $(BASE) && $(MAKE) -f simple.makefile

I_subsys:
	cd $(INVS) && $(MAKE) -f simple.makefile

L_subsys:
	cd $(LAWS) && $(MAKE) -f simple.makefile

N_subsys:
	cd $(LINA) && $(MAKE) -f simple.makefile

M_subsys:
	cd $(MODS) && $(MAKE) -f simple.makefile

P_subsys:
	cd $(PRMS) && $(MAKE) -f simple.makefile

clean:
	cd $(BASE) && $(MAKE) -f simple.makefile clean
	cd $(INVS) && $(MAKE) -f simple.makefile clean
	cd $(LAWS) && $(MAKE) -f simple.makefile clean
	cd $(LINA) && $(MAKE) -f simple.makefile clean
	cd $(MODS) && $(MAKE) -f simple.makefile clean
	cd $(PRMS) && $(MAKE) -f simple.makefile clean
	rm -f *.o

kill:
	cd $(BASE) && $(MAKE) -f simple.makefile clean
	cd $(INVS) && $(MAKE) -f simple.makefile clean
	cd $(LAWS) && $(MAKE) -f simple.makefile clean
	cd $(LINA) && $(MAKE) -f simple.makefile kill
	cd $(MODS) && $(MAKE) -f simple.makefile clean
	cd $(PRMS) && $(MAKE) -f simple.makefile clean
	rm -f *.o $(PROG) $(PROG1)
